import {useCallback, useRef} from 'react';
import {Gesture} from 'react-native-gesture-handler';
import {
  runOnJS,
  useAnimatedStyle,
  useSharedValue,
  withTiming
} from 'react-native-reanimated';

import type {
  GestureStateChangeEvent,
  GestureUpdateEvent,
  PanGestureHandlerEventPayload,
  PinchGestureHandlerEventPayload
} from 'react-native-gesture-handler';

import type {ImageZoomUseGesturesProps} from './types/useGesturesProps';

const clamp = (value: number, min: number, max: number): number => {
  'worklet';

  return Math.min(Math.max(min, value), max);
};

export const useGestures = ({
  center,
  minScale = 1,
  maxScale = 5,
  minPanPointers = 2,
  maxPanPointers = 2,
  isPanEnabled = true,
  isPinchEnabled = true,
  onInteractionStart,
  onInteractionEnd,
  onPinchStart,
  onPinchEnd,
  onPanStart,
  onPanEnd
}: ImageZoomUseGesturesProps) => {
  const isInteracting = useRef(false);
  const isPanning = useRef(false);
  const isPinching = useRef(false);

  const scale = useSharedValue(1);
  const initialFocal = {x: useSharedValue(0), y: useSharedValue(0)};
  const focal = {x: useSharedValue(0), y: useSharedValue(0)};
  const translate = {x: useSharedValue(0), y: useSharedValue(0)};

  const reset = useCallback(() => {
    'worklet';
    scale.value = withTiming(1);
    initialFocal.x.value = 0;
    initialFocal.y.value = 0;
    focal.x.value = withTiming(0);
    focal.y.value = withTiming(0);
    translate.x.value = withTiming(0);
    translate.y.value = withTiming(0);
  }, [
    focal.x,
    focal.y,
    initialFocal.x,
    initialFocal.y,
    scale,
    translate.x,
    translate.y
  ]);

  const onInteractionStarted = () => {
    if (!isInteracting.current) {
      isInteracting.current = true;
      onInteractionStart?.();
    }
  };

  const onInteractionEnded = () => {
    if (isInteracting.current && !isPinching.current && !isPanning.current) {
      reset();
      isInteracting.current = false;
      onInteractionEnd?.();
    }
  };

  const onPinchStarted = () => {
    onInteractionStarted();
    isPinching.current = true;
    onPinchStart?.();
  };

  const onPinchEnded = () => {
    isPinching.current = false;
    onPinchEnd?.();
    onInteractionEnded();
  };

  const onPanStarted = () => {
    onInteractionStarted();
    isPanning.current = true;
    onPanStart?.();
  };

  const onPanEnded = () => {
    isPanning.current = false;
    onPanEnd?.();
    onInteractionEnded();
  };

  const panGesture = Gesture.Pan()
    .enabled(isPanEnabled)
    .minPointers(minPanPointers)
    .maxPointers(maxPanPointers)
    .onStart(() => {
      runOnJS(onPanStarted)();
    })
    .onUpdate((event: GestureUpdateEvent<PanGestureHandlerEventPayload>) => {
      translate.x.value = event.translationX;
      translate.y.value = event.translationY;
    })
    .onEnd(() => {
      runOnJS(onPanEnded)();
    });

  const pinchGesture = Gesture.Pinch()
    .enabled(isPinchEnabled)
    .onStart(
      (event: GestureStateChangeEvent<PinchGestureHandlerEventPayload>) => {
        runOnJS(onPinchStarted)();
        initialFocal.x.value = event.focalX;
        initialFocal.y.value = event.focalY;
      }
    )
    .onUpdate((event: GestureUpdateEvent<PinchGestureHandlerEventPayload>) => {
      scale.value = clamp(event.scale, minScale, maxScale);
      focal.x.value = (center.x - initialFocal.x.value) * (scale.value - 1);
      focal.y.value = (center.y - initialFocal.y.value) * (scale.value - 1);
    })
    .onEnd(() => {
      runOnJS(onPinchEnded)();
    });

  const animatedStyle = useAnimatedStyle(() => ({
    transform: [
      {translateX: translate.x.value},
      {translateY: translate.y.value},
      {translateX: focal.x.value},
      {translateY: focal.y.value},
      {scale: scale.value}
    ]
  }));

  const gestures = Gesture.Simultaneous(pinchGesture, panGesture);

  return {gestures, animatedStyle};
};
